@@ -15,7 +15,7 @@
 ;;  Add (load "~/quilt.el") to your .emacs file
 
 ;;; History:
-;; 
+;;
 
 ;;; Code:
 
@@ -29,12 +29,16 @@
   "Return t if there is on the bottom of patch stack, return nil if otherwise."
   (if (> (call-process "quilt" nil nil nil "applied") 0) 1))
 
-(defun quilt-patches-directory ()
-  "Return the location of patch files."
+(defun quilt--get-config-variable (var)
+  "Return the value of a configuration variable. Return nil if it is unset."
   (or (with-current-buffer (generate-new-buffer " *cmd")
         (shell-command
-         (concat "test -f ~/.quiltrc && . ~/.quiltrc ;"
-                 "echo -n $QUILT_PATCHES")
+         (concat "if [ -f ~/.quiltrc ]; then"
+                 "  . ~/.quiltrc ;"
+                 "elif [ -f /etc/quilt.quiltrc ]; then"
+                 "  . /etc/quilt.quiltrc ;"
+                 "fi ;"
+                 "echo -n $" var)
          t)
         (unwind-protect
             (let ((v (buffer-string)))
@@ -42,17 +46,41 @@
                   nil
                 v))
           (kill-buffer (current-buffer))))
-      (or (getenv "QUILT_PATCHES")
-          "patches")))
+      (getenv var)))
 
-(defun quilt-find-dir (fn)
+(defun quilt--per-project-patches-directory ()
+  (let ((qd (quilt-dir)))
+    (if qd
+        (let ((project-config (concat qd "/"
+                                      (quilt-pc-directory) "/.quilt_patches")))
+          (if (file-readable-p project-config)
+              (with-temp-buffer
+                (insert-file-contents-literally project-config)
+                (substring (buffer-string) 0 -1)))))))
+
+(defun quilt-patches-directory ()
+  "Return the location of patch files."
+  (or (quilt--per-project-patches-directory)
+      (quilt--get-config-variable "QUILT_PATCHES")
+      "patches"))
+
+(defun quilt-pc-directory ()
+  "Return the location of the quilt working state directory."
+  (or (quilt--get-config-variable "QUILT_PC")
+      ".pc"))
+
+(defun quilt-patches-prefix ()
+  "Return the value of the QUILT_PATCHES_PREFIX config variable. Return nil if it is unset."
+  (quilt--get-config-variable "QUILT_PATCHES_PREFIX"))
+
+(defun quilt-find-dir (fn &optional prefn)
   "Return the top level dir of quilt from FN."
-  (if (or (not fn) (equal fn "/"))
+  (if (or (not fn) (equal fn "/") (equal fn prefn))
       nil
     (let ((d (file-name-directory fn)))
-      (if (file-accessible-directory-p (concat d "/.pc"))
+      (if (file-accessible-directory-p (concat d "/" (quilt-pc-directory)))
 	  d
-	(quilt-find-dir (directory-file-name d))))))
+	(quilt-find-dir (directory-file-name d) d)))))
 
 (defun quilt-dir (&optional fn)
   "Return the top level dir of quilt from FN.  FN is just a hint and find from other way if FN is nil."
@@ -84,7 +112,7 @@
       (and
        (not (string-match "\\(~$\\|\\.rej$\\)" fn))
        (not (equal pd (quilt-patches-directory)))
-       (not (equal pd ".pc"))
+       (not (equal pd (quilt-pc-directory)))
        (quilt-p fn)))))
 
 (defun quilt-cmd (cmd &optional buf)
@@ -126,7 +154,7 @@
 
 (defun quilt-patch-list ()
   "Return the list of the name of patches."
-  (quilt-cmd-to-list "seriess"))
+  (quilt-cmd-to-list "series"))
 
 (defun quilt-files-affected (&optional first last)
   "Return the file names which modified from FIRST to LAST."
@@ -152,8 +180,7 @@
   "Return the top patch name.  return nil if there is the bottom of patch stack."
   (if (quilt-bottom-p)
       nil
-    (file-name-nondirectory
-     (substring (quilt-cmd-to-string "top")  0 -1))))
+    (substring (quilt-cmd-to-string "top")  0 -1)))
 
 (defun quilt-complete-list (p l)
   "Call 'completing-read' with prompt P and list L."
@@ -166,6 +193,12 @@
       (setq n (1+ n))))
   (completing-read p l nil t))
 
+(defun quilt--strip-patchname (pn)
+  "Return the name of patch PN sans the path to the patches directory."
+  (let ((patches-path (concat (quilt-patches-directory) "/")))
+    (substring pn (length patches-path))))
+
+(defvar quilt-edit-top-only 't)
 (defun quilt-editable (f)
   "Return t if F is editable in terms of current patch.  Return nil if otherwise."
   (let ((qd (quilt-dir))
@@ -175,20 +208,23 @@
 	(if (quilt-bottom-p)
 	    (quilt-cmd "applied")	; to print error message
 	  (setq dirs (if quilt-edit-top-only
-			 (list (quilt-top-patch))
-			 (cdr (cdr (directory-files (concat qd ".pc/"))))))
+			 (list (let ((patch (substring (quilt-cmd-to-string "top")  0 -1)))
+				 (if (quilt-patches-prefix)
+				     (quilt--strip-patchname patch)
+				   patch)))
+			 (cdr (cdr (directory-files (concat qd (quilt-pc-directory) "/"))))))
 	  (while (and (not result) dirs)
-	    (if (file-exists-p (concat qd ".pc/" (car dirs) "/" fn))
+	    (if (file-exists-p (concat qd (quilt-pc-directory) "/" (car dirs) "/" fn))
 		(setq result t)
 	      (setq dirs (cdr dirs))))
 	  result))))
 
 (defun quilt-short-patchname ()
-  "Return shortened name of top patch.  Return nil if there is on the bottom of patch stack."
+  "Return shortened name of top patch.  Return \"none\" when on the bottom patch stack."
   (let ((p (quilt-top-patch)))
     (if (not p)
 	"none"
-      (let ((p2 (file-name-sans-extension p)))
+      (let ((p2 (file-name-sans-extension (file-name-nondirectory p))))
 	   (if (< (length p2) 10)
 	       p2
 	     (concat (substring p2 0 8) ".."))))))
@@ -204,7 +240,7 @@
   (force-mode-line-update))
 
 (defun quilt-revert (filelist)
-  "Refresh contents, editability and modeline of FILESIT.
+  "Refresh contents, editability and modeline of FILELIST.
 FILELIST won't be touched unless their file is a child of the
 current quilt directory.  Each elements in FILELIST should be the absolute
 file names of those files affected by the latest quilt
@@ -280,7 +316,7 @@ editability adjusted."
 	    (message "no patch name is supplied")
 	  (quilt-save)
 	  (let (cmd first last)
-	    (if (file-exists-p (concat qd ".pc/" arg))
+	    (if (file-exists-p (concat qd (quilt-pc-directory) "/" arg))
 		(progn
 		  (setq cmd "pop")
 		  (setq first (car (quilt-cmd-to-list (concat "next " arg))))
@@ -375,7 +411,7 @@ editability adjusted."
 			    patch)))
 	    (if (file-exists-p pf)
 		(progn (find-file pf)
-		       (toggle-read-only))
+		       (read-only-mode 0))
 	      (message (format "%s doesn't exist yet." pf)))))))))
 
 (defun quilt-patches ()
@@ -537,7 +573,6 @@ editability adjusted."
 
 (defvar quilt-mode nil)
 (make-variable-buffer-local 'quilt-mode)
-(defvar quilt-edit-top-only 't)
 
 (defun quilt-mode (&optional arg)
   "Toggle 'quilt-mode'.  Enable 'quilt-mode' if ARG is positive.
@@ -552,8 +587,8 @@ editability adjusted."
       (let ((f (quilt-buffer-file-name-safe)))
 	(if (quilt-owned-p f)
 	    (if (not (quilt-editable f))
-		(toggle-read-only 1)
-	      (toggle-read-only 0)))
+            (read-only-mode 1)
+	      (read-only-mode 0)))
 	(quilt-update-modeline))))
 
 (defun quilt-hook ()
